package at.ac.uibk.gitsearch.service.gitlab_events;

import at.ac.uibk.gitsearch.service.gitlab_events.GitlabEventService.ItemPath;
import java.util.ArrayList;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.stream.Collectors;
import org.gitlab4j.api.models.Project;
import org.gitlab4j.api.models.RepositoryFile;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A simple tree structure for representing a metadata collection
 *
 * @author Micha
 *
 */
@SuppressWarnings("PMD.TooManyMethods")
class CollectionNode {

    private static final Logger logger = LoggerFactory.getLogger(CollectionNode.class);

    private final ItemPath item;
    private final List<CollectionNode> children;
    private final MetadataInfo metaDataInfo;
    private final RepositoryFile repositoryFile;
    private final CollectionNode parent;
    private final Project gitLabProject;
    private final int level;

    CollectionNode(ItemPath item, MetadataInfo metaDataInfo, RepositoryFile file, CollectionNode parent, Project project) {
        this.item = item;
        this.metaDataInfo = metaDataInfo;
        this.children = new ArrayList<>();
        this.parent = parent;
        this.gitLabProject = project;
        this.level = (parent == null) ? 0 : parent.level + 1;
        this.repositoryFile = file;
    }

    /**
     * Returns a list of all Nodes' paths reachable from this node (the tree having
     * this node
     * as the root flattened into a list).
     *
     * @return all Nodes' paths reachable from this node
     */
    public List<ItemPath> getAllPaths() {
        List<ItemPath> allPaths = new ArrayList<>();
        allPaths.add(item);
        for (CollectionNode child : children) {
            allPaths.addAll(child.getAllPaths());
        }
        return allPaths;
    }

    /**
     * Returns a list containing paths of all direct child nodes of this node.
     *
     * @return the paths of all direct child nodes
     */
    public List<ItemPath> getChildPaths() {
        return children.stream().map(child -> child.item).collect(Collectors.toList());
    }

    @Override
    public String toString() {
        return "Node [" + item.toString() + ", children=" + toStringChildren() + "]";
    }

    private String toStringChildren() {
        return children.stream().map(Object::toString).collect(Collectors.joining(", "));
    }

    public void printTreeRecursive() {
        printTreeRecursive("");
    }

    /**
     * Used internally to recursively generate a visual representation of the tree.
     *
     * @param level distance from the root
     */
    private void printTreeRecursive(String indentation) {
        logger.info("{}{}", indentation, this.item.getFullPath());
        for (CollectionNode child : children) {
            child.printTreeRecursive(indentation + " ");
        }
    }

    public boolean isRoot() {
        return level == 0;
    }

    public RepositoryFile getRepositoryFile() {
        return repositoryFile;
    }

    public Project getGitLabProject() {
        if (gitLabProject == null && parent != null) {
            return parent.getGitLabProject();
        } else {
            return gitLabProject;
        }
    }

    public ItemPath getItem() {
        return item;
    }

    public List<CollectionNode> getChildren() {
        return children;
    }

    public CollectionNode getParent() {
        return parent;
    }

    public int getLevel() {
        return level;
    }

    public MetadataInfo getMetaDataInfo() {
        return metaDataInfo;
    }

    public boolean hasErrors() {
        return !metaDataInfo.getErrors().isEmpty() || this.getChildren().stream().anyMatch(node -> node.hasErrors());
    }

    public boolean hasWarnings() {
        return !metaDataInfo.getWarnings().isEmpty() || this.getChildren().stream().anyMatch(node -> node.hasWarnings());
    }

    public SortedMap<String, List<String>> getErrors() {
        SortedMap<String, List<String>> collector = new TreeMap<>();
        collectErrors(collector);
        return collector;
    }

    private void collectErrors(SortedMap<String, List<String>> collector) {
        collector.put(this.item.toString(), metaDataInfo.getErrors());
        this.getChildren().stream().forEach(node -> node.collectErrors(collector));
    }

    public SortedMap<String, List<String>> getWarnings() {
        SortedMap<String, List<String>> collector = new TreeMap<>();
        collectWarnings(collector);
        return collector;
    }

    private void collectWarnings(SortedMap<String, List<String>> collector) {
        collector.put(this.item.toString(), metaDataInfo.getWarnings());
        this.getChildren().stream().forEach(node -> node.collectWarnings(collector));
    }
}
