package at.ac.uibk.gitsearch.web.rest;

import at.ac.uibk.gitsearch.repository.search.ElasticSearchRepository;
import at.ac.uibk.gitsearch.service.gitlab_events.GitlabEventService;
import at.ac.uibk.gitsearch.service.gitlab_events.LogHolder;
import co.elastic.clients.elasticsearch.cat.aliases.AliasesRecord;
import co.elastic.clients.elasticsearch.cat.indices.IndicesRecord;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/api/esManagement")
public class ElasticSearchManagementResource {

    @SuppressWarnings("unused")
    private static final Logger log = LoggerFactory.getLogger(ElasticSearchManagementResource.class);

    @Autowired
    private ElasticSearchRepository elasticSearchRepository;

    @Autowired
    private GitlabEventService gitlabEventService;

    private final Map<String, LogHolder> indexLoggerMap = new ConcurrentHashMap<>();

    /**
     * returns all gitlab indices.
     *
     */
    @GetMapping("/indices")
    public List<IndicesRecordWrapper> getIndices() throws IOException {
        return elasticSearchRepository.getIndices().stream().map(IndicesRecordWrapper::new).collect(Collectors.toList());
    }

    /**
     * returns all gitlab indices.
     *
     */
    @DeleteMapping("/index/{index}")
    public boolean deleteIndex(@PathVariable String index) throws IOException {
        return elasticSearchRepository.deleteIndex(index);
    }

    /**
     * returns all aliases.
     *
     */
    @GetMapping("/aliases")
    public List<AliasesRecordWrapper> getAliases() throws IOException {
        return elasticSearchRepository.getAliases().stream().map(AliasesRecordWrapper::new).collect(Collectors.toList());
    }

    static class IncrementalLoggerInfo {

        private final LogHolder log;

        protected IncrementalLoggerInfo(LogHolder log) {
            super();
            this.log = log;
        }

        public String getLog() {
            return log.readAndDelete();
        }

        public boolean isClosed() {
            return log.isClosed();
        }
    }

    static class ErrorLoggerInfo extends IncrementalLoggerInfo {

        String info;

        protected ErrorLoggerInfo(String info) {
            super(null);
            this.info = info;
        }

        @Override
        public String getLog() {
            return "No Log found: " + info;
        }

        @Override
        public boolean isClosed() {
            return true;
        }
    }

    @PostMapping("/reindex")
    public String reindex() throws IOException {
        LogHolder indexAllProjectsAsync = gitlabEventService.indexAllProjectsAsync();

        String uuidString = indexAllProjectsAsync.getId().toString();

        indexLoggerMap.put(uuidString, indexAllProjectsAsync);

        return "\"" + uuidString + "\""; // just a trivial json string
    }

    @GetMapping("/reindexLog/{logId}")
    public IncrementalLoggerInfo getIndexLog(@PathVariable String logId) throws IOException {
        LogHolder logHolder = indexLoggerMap.get(logId);
        if (logHolder == null) {
            return new ErrorLoggerInfo(logId);
        }
        if (logHolder.isClosed()) {
            indexLoggerMap.remove(logId);
        }
        return new IncrementalLoggerInfo(logHolder);
    }

    /**
     * this is a simple wrapper class to enable correct json deserialisation.
     *
     * @author Michael Breu
     *
     */
    protected static class IndicesRecordWrapper {

        protected IndicesRecord indexRecord;

        protected IndicesRecordWrapper(IndicesRecord indexRecord) {
            super();
            this.indexRecord = indexRecord;
        }

        public final String getStatus() {
            return indexRecord.status();
        }

        public final String getUuid() {
            return indexRecord.uuid();
        }

        public final String getDocsCount() {
            return indexRecord.docsCount();
        }

        public final String getStoreSize() {
            return indexRecord.storeSize();
        }

        @Override
        public String toString() {
            return indexRecord.toString();
        }

        public String getHealth() {
            return indexRecord.health();
        }

        public String getIndex() {
            return indexRecord.index();
        }

        public String getDocsDeleted() {
            return indexRecord.docsDeleted();
        }

        public String getCreationDate() {
            return indexRecord.creationDate();
        }

        public String getCreationDateString() {
            return indexRecord.creationDateString();
        }
        //        public IndicesRecord getIndexRecord() {
        //            return indexRecord;
        //        }

        //        public void setIndexRecord(IndicesRecord indexRecord) {
        //            this.indexRecord = indexRecord;
        //        }
    }

    /**
     * this is a simple wrapper class to enable correct json deserialisation.
     *
     * @author Micha
     *
     */
    protected class AliasesRecordWrapper {

        private final AliasesRecord aliasRecord;

        protected AliasesRecordWrapper(AliasesRecord aliasRecord) {
            super();
            this.aliasRecord = aliasRecord;
        }

        public final String getIndex() {
            return aliasRecord.index();
        }

        public final String getFilter() {
            return aliasRecord.filter();
        }

        public final String getRoutingIndex() {
            return aliasRecord.routingIndex();
        }

        public final String getRoutingSearch() {
            return aliasRecord.routingSearch();
        }

        public final String getIsWriteIndex() {
            return aliasRecord.isWriteIndex();
        }

        public final String getAlias() {
            return aliasRecord.alias();
        }
    }
}
