package at.ac.uibk.gitsearch.web.rest;

import at.ac.uibk.gitsearch.repository.jpa.SavedSearchesRepository;
import at.ac.uibk.gitsearch.service.SavedSearchesService;
import at.ac.uibk.gitsearch.service.dto.SavedSearchesDTO;
import at.ac.uibk.gitsearch.web.rest.errors.BadRequestAlertException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.PaginationUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link at.ac.uibk.gitsearch.domain.SavedSearches}.
 */
@RestController
@RequestMapping("/api")
public class SavedSearchesResource {

    private final Logger log = LoggerFactory.getLogger(SavedSearchesResource.class);

    private static final String ENTITY_NAME = "savedSearches";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final SavedSearchesService savedSearchesService;

    private final SavedSearchesRepository savedSearchesRepository;

    public SavedSearchesResource(SavedSearchesService savedSearchesService, SavedSearchesRepository savedSearchesRepository) {
        this.savedSearchesService = savedSearchesService;
        this.savedSearchesRepository = savedSearchesRepository;
    }

    /**
     * {@code POST  /saved-searches} : Create a new savedSearches.
     *
     * @param savedSearchesDTO the savedSearchesDTO to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new savedSearchesDTO, or with status {@code 400 (Bad Request)} if the savedSearches has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("/saved-searches")
    public ResponseEntity<SavedSearchesDTO> createSavedSearches(@Valid @RequestBody SavedSearchesDTO savedSearchesDTO)
        throws URISyntaxException {
        log.debug("REST request to save SavedSearches : {}", savedSearchesDTO);
        if (savedSearchesDTO.getId() != null) {
            throw new BadRequestAlertException("A new savedSearches cannot already have an ID", ENTITY_NAME, "idexists");
        }
        SavedSearchesDTO result = savedSearchesService.save(savedSearchesDTO);
        return ResponseEntity
            .created(new URI("/api/saved-searches/" + result.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, result.getId().toString()))
            .body(result);
    }

    /**
     * {@code PUT  /saved-searches/:id} : Updates an existing savedSearches.
     *
     * @param id the id of the savedSearchesDTO to save.
     * @param savedSearchesDTO the savedSearchesDTO to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated savedSearchesDTO,
     * or with status {@code 400 (Bad Request)} if the savedSearchesDTO is not valid,
     * or with status {@code 500 (Internal Server Error)} if the savedSearchesDTO couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @SuppressWarnings("PMD.AvoidDuplicateLiterals")
    @PutMapping("/saved-searches/{id}")
    public ResponseEntity<SavedSearchesDTO> updateSavedSearches(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody SavedSearchesDTO savedSearchesDTO
    ) throws URISyntaxException {
        log.debug("REST request to update SavedSearches : {}, {}", id, savedSearchesDTO);
        if (savedSearchesDTO.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, savedSearchesDTO.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!savedSearchesRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        SavedSearchesDTO result = savedSearchesService.save(savedSearchesDTO);
        return ResponseEntity
            .ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, savedSearchesDTO.getId().toString()))
            .body(result);
    }

    /**
     * {@code PATCH  /saved-searches/:id} : Partial updates given fields of an existing savedSearches, field will ignore if it is null
     *
     * @param id the id of the savedSearchesDTO to save.
     * @param savedSearchesDTO the savedSearchesDTO to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated savedSearchesDTO,
     * or with status {@code 400 (Bad Request)} if the savedSearchesDTO is not valid,
     * or with status {@code 404 (Not Found)} if the savedSearchesDTO is not found,
     * or with status {@code 500 (Internal Server Error)} if the savedSearchesDTO couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/saved-searches/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<SavedSearchesDTO> partialUpdateSavedSearches(
        @PathVariable(value = "id", required = false) final Long id,
        @NotNull @RequestBody SavedSearchesDTO savedSearchesDTO
    ) throws URISyntaxException {
        log.debug("REST request to partial update SavedSearches partially : {}, {}", id, savedSearchesDTO);
        if (savedSearchesDTO.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, savedSearchesDTO.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!savedSearchesRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<SavedSearchesDTO> result = savedSearchesService.partialUpdate(savedSearchesDTO);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, savedSearchesDTO.getId().toString())
        );
    }

    /**
     * {@code GET  /saved-searches} : get all the savedSearches.
     *
     * @param pageable the pagination information.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of savedSearches in body.
     */
    @GetMapping("/saved-searches")
    public ResponseEntity<List<SavedSearchesDTO>> getAllSavedSearches(@org.springdoc.api.annotations.ParameterObject Pageable pageable) {
        log.debug("REST request to get a page of SavedSearches");
        Page<SavedSearchesDTO> page = savedSearchesService.findAll(pageable);
        HttpHeaders headers = PaginationUtil.generatePaginationHttpHeaders(ServletUriComponentsBuilder.fromCurrentRequest(), page);
        return ResponseEntity.ok().headers(headers).body(page.getContent());
    }

    /**
     * {@code GET  /saved-searches/:id} : get the "id" savedSearches.
     *
     * @param id the id of the savedSearchesDTO to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the savedSearchesDTO, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/saved-searches/{id}")
    public ResponseEntity<SavedSearchesDTO> getSavedSearches(@PathVariable Long id) {
        log.debug("REST request to get SavedSearches : {}", id);
        Optional<SavedSearchesDTO> savedSearchesDTO = savedSearchesService.findOne(id);
        return ResponseUtil.wrapOrNotFound(savedSearchesDTO);
    }

    /**
     * {@code DELETE  /saved-searches/:id} : delete the "id" savedSearches.
     *
     * @param id the id of the savedSearchesDTO to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/saved-searches/{id}")
    public ResponseEntity<Void> deleteSavedSearches(@PathVariable Long id) {
        log.debug("REST request to delete SavedSearches : {}", id);
        savedSearchesService.delete(id);
        return ResponseEntity
            .noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
